@@ -1,22 +1,63 @@
-(* $Id: pxp_reader.ml,v 1.26 2003/06/21 13:17:30 gerd Exp $
+(* $Id: pxp_reader.ml 742 2009-02-02 18:03:12Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
  *)
 
-open Pxp_core_types;;
+open Pxp_core_types.I;;
 open Netchannels;;
 
-exception Not_competent;;
-exception Not_resolvable of exn;;
+exception Not_competent = Pxp_core_types.I.Not_competent;;
+exception Not_resolvable (* of exn *) = Pxp_core_types.I.Not_resolvable;;
+
+type lexer_source =
+    { lsrc_lexbuf : Lexing.lexbuf Lazy.t;
+      lsrc_unicode_lexbuf : Netulex.ULB.unicode_lexbuf Lazy.t;
+    }
+
+
+
+let ensure_space_minimum p f g =
+  (* The functional [f] is called with a function as argument that refills
+   * a buffer string (think f = Lexing.from_function). This argument is [g],
+   * i.e. in most cases this is the same as [f g]. However, it is ensured
+   * that the string buffer has the minimum free space [p]. This is achieved
+   * by using an auxiliary buffer.
+   *)
+  let buf = String.create p in
+  let bufpos = ref 0 in
+  let buflen = ref 0 in
+  f (fun s n ->
+       assert(n>0);
+       if !buflen > 0 then (
+	 let m = min n !buflen in
+	 String.blit buf !bufpos s 0 m;
+	 bufpos := !bufpos + m;
+	 buflen := !buflen - m;
+	 m
+       )
+       else
+	 if n < p then (
+	   let l = g buf p in
+	   let m = min l n in
+	   String.blit buf 0 s 0 m;
+	   bufpos := m;
+	   buflen := l-m;
+	   m
+	 )
+	 else 
+	   g s n
+    )
+;;
+
 
 class type resolver =
   object
     method init_rep_encoding : rep_encoding -> unit
     method init_warner : symbolic_warnings option -> collect_warnings -> unit
     method rep_encoding : rep_encoding
-    method open_in : ext_id -> Lexing.lexbuf
-    method open_rid : resolver_id -> Lexing.lexbuf
+    method open_in : ext_id -> lexer_source
+    method open_rid : resolver_id -> lexer_source
     method active_id : resolver_id
     method close_in : unit
     (* method close_all : unit *)
@@ -38,8 +79,14 @@ class virtual resolve_general
 
     val mutable is_open = false
 
+    val mutable mode = None
+		(* Whether the [lexbuf_reader] or the [unicode_lexbuf_reader]
+		 * is used. One can only invoke one of them.
+		 *)
+
     val mutable encoding = `Enc_utf8
     val mutable encoding_requested = false
+    val mutable encoding_request_post = (fun () -> ()) (* post action *)
 
     val mutable active_id = null_resolver
 
@@ -49,9 +96,6 @@ class virtual resolve_general
     val mutable enc_initialized = false
     val mutable wrn_initialized = false
 
-(* (* needed to support close_all: *)
-    val mutable clones = []
-*)
 
     method init_rep_encoding e =
       internal_encoding <- e;
@@ -64,13 +108,6 @@ class virtual resolve_general
 
     method rep_encoding = (internal_encoding :> rep_encoding)
 
-(*
-    method clone =
-      ( {< encoding = `Enc_utf8;
-	   encoding_requested = false;
-	>}
-	: # resolver :> resolver )
-*)
 
     method private warn (k:int) =
       (* Called if a character not representable has been found.
@@ -91,17 +128,36 @@ class virtual resolve_general
        * "UTF-16-BE": UTF-16/UCS-2 encoding big endian
        * "UTF-16-LE": UTF-16/UCS-2 encoding little endian
        * "UTF-8":     UTF-8 encoding
+       *
+       * Returns the number of bytes to eat up in the buffer
        *)
-      if String.length s < 4 then
-	encoding <- `Enc_utf8
-      else if String.sub s 0 2 = "\254\255" then
-	encoding <- `Enc_utf16
-	  (* Note: Netconversion.recode will detect the big endianess, too *)
-      else if String.sub s 0 2 = "\255\254" then
-	encoding <- `Enc_utf16
-	  (* Note: Netconversion.recode will detect the little endianess, too *)
-      else
-	encoding <- `Enc_utf8
+      if String.sub s 0 2 = "\254\255" then (
+	encoding <- `Enc_utf16_be;
+	2
+      )
+      else if String.sub s 0 4 = "\000\060\000\063" then (
+	encoding <- `Enc_utf16_be;
+	0
+      )
+      else if String.sub s 0 2 = "\255\254" then (
+	encoding <- `Enc_utf16_le;
+	2
+      )
+      else if String.sub s 0 4 = "\060\000\063\000" then (
+	encoding <- `Enc_utf16_le;
+	0
+      )
+      else if String.sub s 0 3 = "\239\187\191" then (
+	(* That's the unusual case of a byte order mark in UTF-8 encoding.
+           This is not mentioned in the XML standard, but Unicode allows it.
+	 *)
+	encoding <- `Enc_utf8;
+	3
+      )
+      else (
+	encoding <- `Enc_utf8;
+	0
+      )
 
 
     method private virtual next_string : string -> int -> int -> int
@@ -109,10 +165,243 @@ class virtual resolve_general
     method virtual close_in : unit
       (* must reset is_open! *)
 
-(*
-    method close_all =
-      List.iter (fun r -> r # close_in) clones
-*)
+
+    method private lexbuf_reader () =
+      if mode <> None && mode <> Some `Lexbuf then 
+	failwith "lexbuf_reader: other reader already working";
+
+      mode <- Some `Lexbuf;
+      let direct_reader = ref false in  (* whether to bypass the buffer *)
+
+      let buf_max = 4096 in
+      let buf     = ref (String.make buf_max ' ')  in
+      let buf_beg = ref 0 in
+      let buf_end = ref 0 in
+      let buf_eof = ref false in
+      (* The buffer is used if [not direct_reader]. [buf_beg] is the 
+       * beginning of the filled part of the buffer, [buf_end] is the
+       * end (plus 1). [buf_eof] indicates that EOF was already signaled,
+       * and should be processed after the current contents.
+       *)
+
+      let refill() =
+	(* Refill the buffer from [next_string].
+	 * It is important to refill as much as possible for the
+	 * algorithm below.
+	 *)
+	while not !buf_eof && !buf_end < buf_max do
+	  let n = self # next_string !buf !buf_end (buf_max - !buf_end) in
+	  if n=0 then
+	    buf_eof := true
+	  else
+	    buf_end := !buf_end + n
+	done
+      in
+
+      let convert s n max_chars =
+	(* Convert characters from [buf] to [s], [n]. The number of characters
+	 * it limited by [max_chars]
+	 *)
+	assert(n>=6);
+	if !buf_beg + 6 > !buf_end && not !buf_eof then (
+	  (* Less than 6 bytes in [buf]. We need at least this
+	   * number of bytes, which is the length of the longest
+	   * UTF-8 char, otherwise we cannot ensure to convert
+	   * at least one character.
+	   *)
+	  let m = !buf_end - !buf_beg in
+	  String.blit !buf !buf_beg !buf 0 m;
+	  buf_beg := 0;
+	  buf_end := m;
+	  refill();
+	);
+
+	(* It is still possible that there are less than 6 bytes
+	 * in [buf], but only if [buf_eof] is true at the same
+	 * time.
+	 *)
+	
+	let m = !buf_end - !buf_beg in
+	if m=0 then (
+	  assert !buf_eof;
+	  0  (* EOF! *)
+	)
+	else (
+	  let (n_in, n_out, encoding') =
+	    Netconversion.recode
+	      ~in_enc:encoding
+	      ~in_buf:!buf
+	      ~in_pos:!buf_beg
+	      ~in_len:m
+	      ~out_enc:(internal_encoding : rep_encoding :> encoding)
+	      ~out_buf:s
+	      ~out_pos:0
+	      ~out_len:n
+	      ~max_chars
+	      ~subst:(fun k -> self # warn k; "") in
+	  if n_in = 0 then (
+	    (* An incomplete character at the end of the stream. 
+	     * Note: This test assumes that there is one character to 
+	     * convert in [buf], and that there is enough free space in [s].
+	     * This is the case because we ensure [m>=6] and [n>=6].
+	     *)
+	    assert !buf_eof;
+	    raise Netconversion.Malformed_code;
+	  );
+	  encoding <- encoding';
+	  buf_beg := !buf_beg + n_in;
+	  
+	  assert(n_out > 0);
+	  n_out
+	)
+      in
+
+      (* Fill the buffer initially. We start always with buffered reading,
+       * but we try to switch to direct reading later.
+       *)
+      refill();
+      if !buf_end >= 4 && not encoding_requested then (
+	let n_skip = self # autodetect !buf in
+	buf_beg := !buf_beg + n_skip;
+      );
+
+      (* Ensure that [n >= 6], the longest UTF-8 character, so we can always
+       * put at least one character into [s]
+       *)
+      ensure_space_minimum 6
+	Lexing.from_function
+	(fun s n ->
+	   (* Fill the string [s] with at most [n] bytes. Return the number
+	    * of bytes, or 0 to signal EOF.
+	    *)
+	   if not is_open then
+	     failwith "trying to read from resolver, but resolver is not open";
+	   
+	   if !direct_reader then
+	     self # next_string s 0 n 
+	   else (
+	     if encoding_requested then (
+	       (* In this case, the encoding will not change any more. We
+		* can read any number of characters at once.
+		*)
+	       if encoding = (internal_encoding : rep_encoding :> encoding)
+	       then (
+		 (* No conversion is needed. In order to speed up reading,
+		  * we are going to enable [direct_reader].
+		  *)
+		 if !buf_beg < !buf_end then (
+		   (* There are still bytes in [buf], return them first *)
+		   let m = min n (!buf_end - !buf_beg) in
+		   String.blit !buf !buf_beg s 0 m;
+		   buf_beg := !buf_beg + m;
+		   m
+		 )
+		 else (
+		   (* Either we are already at EOF, or we can switch to
+		    * [direct_reader].
+		    *)
+		   buf := "";  (* Free buf, it will never be used again *)
+		   if !buf_eof then
+		     0
+		   else (
+		     direct_reader := true;
+		     self # next_string s 0 n
+		   )
+		 )
+	       )
+	       else (
+		 (* Character conversion is needed from [encoding] to
+		  * [internal_encoding].
+		  *)
+		 convert s n max_int
+	       )
+	     )
+	     else (
+	       (* In this case, the encoding might change at any time.
+		* Because of this, we only read one character at a time.
+		*)
+	       convert s n 1
+	     )
+	   )
+	)
+
+
+    method private unicode_lexbuf_reader () =
+      if mode <> None && mode <> Some `Unicode_lexbuf then 
+	failwith "unicode_lexbuf_reader: other reader already working";
+
+      mode <- Some `Unicode_lexbuf;
+
+      let buf = Netbuffer.create 4 in
+      (* Only used for autodetection! *)
+
+      let buf_eof = ref false in
+
+      if not encoding_requested then (
+	while not !buf_eof && Netbuffer.length buf < 4 do
+	  let n =
+	    Netbuffer.add_inplace ~len:(4-Netbuffer.length buf) buf self#next_string
+	  in
+	  buf_eof := (n=0)
+	done;
+	if Netbuffer.length buf >= 4  then (
+	  let n_skip = self # autodetect (Netbuffer.contents buf) in
+	  Netbuffer.delete buf 0 n_skip
+	)
+      );
+
+      let lexbuf =
+	Netulex.ULB.from_function
+	  ~enc_change_hook:(
+	    fun ulb -> 
+	      encoding <- ulb.Netulex.ULB.ulb_encoding )
+	  ~refill:(
+	    fun s p n ->
+	      (* Fill the string [s] at position [p] with at most [n] bytes. 
+	       * Return the number of bytes, or 0 to signal EOF.
+	       *)
+	      if not is_open then
+		failwith "trying to read from resolver, but resolver is not open";
+	      
+	      if encoding_requested && Netbuffer.length buf = 0 then (
+		(* In this case, the encoding will not change any more. We
+		 * can read any number of characters at once.
+		 *)
+		self # next_string s p n
+	      )
+	      else (
+		(* In this case, the encoding might change at any time.
+		 * Because of this, we only read one character at a time.
+		 * If there are still characters in [buf], take these first.
+		 *)
+		
+		if Netbuffer.length buf > 0 then (
+		  let c = (Netbuffer.contents buf).[0] in
+		  s.[p] <- c;
+		  Netbuffer.delete buf 0 1;
+		  1
+		)
+
+		else (
+		  if !buf_eof then
+		    0 (* EOF already seen *)
+		  else
+		    self # next_string s p 1
+		)
+	      )
+	  )
+	  encoding
+      in
+
+      encoding_request_post <- (
+	fun () ->
+	  (* This function is called after the encoding was requested for
+	   * the first time
+	   *)  
+	  Netulex.ULB.set_encoding encoding lexbuf
+      );
+
+      lexbuf
 
     method open_in xid =
       self # open_rid (resolver_id_of_ext_id xid)
@@ -127,76 +416,9 @@ class virtual resolve_general
 
       is_open <- true;
 
-      let buffer_max = 512 in
-      let buffer = String.make buffer_max ' ' in
-      let buffer_len = ref 0 in
-      let buffer_end = ref false in
-      let fillup () =
-	if not !buffer_end & !buffer_len < buffer_max then begin
-	  let l =
-	    self # next_string buffer !buffer_len (buffer_max - !buffer_len) in
-	  if l = 0 then
-	    buffer_end := true
-	  else begin
-	    buffer_len := !buffer_len + l
-	  end
-	end
-      in
-      let consume n =
-	let l = !buffer_len - n in
-	if l > 0 then String.blit buffer n buffer 0 l;
-	buffer_len := l
-      in
-
-      fillup();
-      if not encoding_requested then self # autodetect buffer;
-
-      Pxp_lexing.from_function
-	(fun s n ->
-	   (* TODO: if encoding = internal_encoding, it is possible to
-	    * avoid copying buffer to s because s can be directly used
-	    * as buffer.
-	    *)
-
-	   if not is_open then
-	     failwith "trying to read from resolver, but resolver is not open";
-
-	   fillup();
-	   if !buffer_len = 0 then
-	     0
-	   else begin
-	     let m_in  = !buffer_len in
-	     let m_max = if encoding_requested then n else 1 in
-	     let n_in, n_out, encoding' =
-	       if encoding = (internal_encoding : rep_encoding :> encoding) &&
-	          encoding_requested
-	       then begin
-		 (* Special case encoding = internal_encoding *)
-		 String.blit buffer 0 s 0 m_in;
-		 m_in, m_in, encoding
-	       end
-	       else
-		 Netconversion.recode
-		   ~in_enc:encoding
-		   ~in_buf:buffer
-		   ~in_pos:0
-		   ~in_len:m_in
-		   ~out_enc:(internal_encoding : rep_encoding :> encoding)
-		   ~out_buf:s
-		   ~out_pos:0
-		   ~out_len:n
-		   ~max_chars:m_max
-		   ~subst:(fun k -> self # warn k; "")
-	     in
-	     if n_in = 0 then
-	       (* An incomplete character at the end of the stream: *)
-	       raise Netconversion.Malformed_code;
-	       (* failwith "Badly encoded character"; *)
-	     encoding <- encoding';
-	     consume n_in;
-	     assert(n_out <> 0);
-	     n_out
-	   end)
+      { lsrc_lexbuf         = lazy(self # lexbuf_reader() );
+	lsrc_unicode_lexbuf = lazy(self # unicode_lexbuf_reader() );
+      }
 
     method change_encoding enc =
       if not is_open then
@@ -216,6 +438,7 @@ class virtual resolve_general
 	end;
 	(* else: the autodetected encoding counts *)
 	encoding_requested <- true;
+	encoding_request_post();
       end;
 
     method active_id = 
@@ -257,7 +480,10 @@ object(self)
       match current_channel with
 	  None -> failwith "Pxp_reader.resolve_read_any_channel # next_string"
 	| Some ch ->
-	    ch # input s ofs len
+	    try
+	      ch # input s ofs len
+	    with
+		End_of_file -> 0
 
     method close_in =
       is_open <- false;
@@ -500,7 +726,8 @@ class resolve_as_file
   ?(system_encoding = `Enc_utf8)
   ?(map_private_id = (fun _ -> raise Not_competent))
   ?(open_private_id = (fun _ -> raise Not_competent))
-  ?(base_url_defaults_to_cwd = true)
+  ?(base_url_defaults_to_cwd = false)
+  ?(not_resolvable_if_not_found = true)
   ()
   =
 
@@ -522,12 +749,18 @@ class resolve_as_file
   let default_base_url =
     if base_url_defaults_to_cwd then begin
       let cwd = Sys.getcwd() in
-      let cwd_slash = if cwd = "/" then cwd else cwd ^ "/" in
-      Some(Neturl.make_url
-	     ~scheme: "file"
-	     ~host:   ""
-	     ~path:   (Neturl.split_path cwd_slash)
-	     base_url_syntax)
+      let cwd_utf8 = 
+	Netconversion.recode_string
+	  ~in_enc: system_encoding
+	  ~out_enc: `Enc_utf8 
+	  cwd in
+      let l = String.length cwd_utf8 in
+      let cwd_utf8 =
+	if cwd_utf8 = "" || cwd_utf8.[l-1] <> '/' then 
+	  cwd_utf8 ^ "/" 
+	else
+	  cwd_utf8 in
+      Some(Neturl.file_url_of_local_path cwd_utf8)
     end
     else
       None
@@ -594,9 +827,9 @@ class resolve_as_file
     else begin
       try
 	let path_utf8 =
-	  try Neturl.join_path (Neturl.url_path ~encoded:false url)
-	  with Not_found -> raise Not_competent
+	  Neturl.local_path_of_file_url url (* may fail *)
 	in
+	(* Note: it is only assumed that the path is UTF-8 *)
 	      
 	let path =
 	  Netconversion.recode_string
@@ -604,13 +837,16 @@ class resolve_as_file
 	    ~out_enc: system_encoding
 	    path_utf8 in
             (* May raise Malformed_code *)
+
+	if (not not_resolvable_if_not_found) && not(Sys.file_exists path) then
+	  raise Not_competent;
 	      
 	(new input_channel(open_in_bin path), None, None)
 	(* May raise Sys_error *)
 		
       with
-	| Netconversion.Malformed_code -> assert false
-  	    (* should not happen *)
+	| Netconversion.Malformed_code as e ->
+	    raise (Not_resolvable e)
 	| Sys_error _ as e ->
 	    raise (Not_resolvable e)
     end
@@ -632,26 +868,17 @@ let make_file_url ?(system_encoding = `Enc_utf8) ?(enc = `Enc_utf8) filename =
       filename
   in
 
-  let utf8_abs_filename =
-    if utf8_filename <> "" && utf8_filename.[0] = '/' then
-      utf8_filename
-    else
-      let cwd = Sys.getcwd() in
-      let cwd_utf8 =
-	Netconversion.recode_string
+  let getcwd() =
+    let cwd = Sys.getcwd() in
+    let cwd_utf8 = 
+      Netconversion.recode_string
 	~in_enc: system_encoding
-	~out_enc: `Enc_utf8 cwd in
-      if cwd = "/" then "/" ^ utf8_filename else cwd_utf8 ^ "/" ^ utf8_filename
+	~out_enc: `Enc_utf8 
+	cwd in
+    cwd_utf8
   in
 
-  let syntax = { Neturl.ip_url_syntax with Neturl.url_accepts_8bits = true } in
-  let url = Neturl.make_url
-	    ~scheme:"file"
-	    ~host:"localhost"
-	    ~path:(Neturl.split_path utf8_abs_filename)
-	      syntax
-  in
-  url
+  Neturl.file_url_of_local_path ~getcwd utf8_filename
 ;;
 
 
@@ -845,9 +1072,8 @@ type combination_mode =
 ;;
 
 
-class combine ?prefer ?mode rl =
+class combine ?mode rl =
   object (self)
-    val prefered_resolver = prefer
     val mode = mode
     val resolvers = (rl : resolver list)
     val mutable internal_encoding = `Enc_utf8
@@ -917,9 +1143,7 @@ class combine ?prefer ?mode rl =
 
       if active_resolver <> None then failwith "Pxp_reader.combine # open_rid";
       let r, lb =
-	match prefered_resolver with
-	    None ->   find_competent_resolver resolvers
-	  | Some r -> find_competent_resolver (r :: resolvers)
+	find_competent_resolver resolvers
       in
       active_resolver <- Some r;
       lb
@@ -949,12 +1173,11 @@ class combine ?prefer ?mode rl =
       let c =
 	match active_resolver with
 	    None   ->
-	      new combine ?prefer:None ?mode
+	      new combine ?mode
                           (List.map (fun q -> q # clone) resolvers)
 	  | Some r ->
 	      let r' = r # clone in
 	      new combine
-		?prefer:(Some r')
 		?mode
 		(List.map
 		   (fun q -> if q == r then r' else q # clone)
@@ -1371,132 +1594,3 @@ let lookup_system_id_as_string ?fixenc catalog =
 
 
 
-(* ======================================================================
- * History:
- *
- * $Log: pxp_reader.ml,v $
- * Revision 1.26  2003/06/21 13:17:30  gerd
- * 	Fixes regarding Anonymous.
- *
- * Revision 1.25  2003/06/20 15:14:14  gerd
- * 	Introducing symbolic warnings, expressed as polymorphic
- * variants
- *
- * Revision 1.24  2003/06/19 21:51:12  gerd
- * 	Removed the close_all method. It is no longer necessary, because
- * we have now the entity_manager.
- * 	Several smaller fixes.
- *
- * Revision 1.23  2003/06/15 12:23:21  gerd
- * 	Moving core type definitions to Pxp_core_types
- *
- * Revision 1.22  2003/02/01 15:02:09  gerd
- * 	New: norm_system_id, rewrite_system_id.
- * 	Fixed: The empty system name is regarded as "NIL" value. It
- * never matches with itself.
- *
- * Revision 1.21  2003/01/21 00:18:48  gerd
- * 	Reimplementation of the reader classes, now basing on the
- * resolver_id type to identify entities.
- *
- * Revision 1.20  2002/03/08 14:37:15  gerd
- * 	Fixed the case that cwd=/
- *
- * Revision 1.19  2002/02/20 00:25:23  gerd
- * 	using Pxp_lexing instead of Lexing.
- *
- * Revision 1.18  2002/02/18 00:25:40  gerd
- * 	Minor enhancement: String.blit only called for n>0
- *
- * Revision 1.17  2001/10/12 21:38:14  gerd
- * 	Changes for O'caml 3.03-alpha.
- *
- * Revision 1.16  2001/07/01 09:46:40  gerd
- * 	Fix: resolve_read_url_channel does not use the base_url if
- * the current URL is already absolute
- *
- * Revision 1.15  2001/07/01 08:35:23  gerd
- * 	Instead of the ~auto_close argument, there is now a
- * ~close argument for several functions/classes. This allows some
- * additional action when the resolver is closed.
- *
- * Revision 1.14  2001/06/14 23:28:02  gerd
- * 	Fix: class combine works now with private IDs.
- *
- * Revision 1.13  2001/04/22 14:16:48  gerd
- * 	resolve_as_file: you can map private IDs to arbitrary channels.
- * 	resolve_read_url_channel: changed type of the channel_of_url
- * argument (ext_id is also passed)
- * 	More examples and documentation.
- *
- * Revision 1.12  2001/04/21 17:40:48  gerd
- * 	Bugfix in 'combine'
- *
- * Revision 1.11  2001/04/03 20:22:44  gerd
- * 	New resolvers for catalogs of PUBLIC and SYSTEM IDs.
- * 	Improved "combine": PUBLIC and SYSTEM IDs are handled
- * separately.
- * 	Rewritten from_file: Is now a simple application of the
- * Pxp_reader classes and functions. (The same has still to be done
- * for from_channel!)
- *
- * Revision 1.10  2001/02/01 20:38:49  gerd
- * 	New support for PUBLIC identifiers.
- *
- * Revision 1.9  2000/08/14 22:24:55  gerd
- * 	Moved the module Pxp_encoding to the netstring package under
- * the new name Netconversion.
- *
- * Revision 1.8  2000/07/16 18:31:09  gerd
- * 	The exception Illegal_character has been dropped.
- *
- * Revision 1.7  2000/07/09 15:32:01  gerd
- * 	Fix in resolve_this_channel, resolve_this_string
- *
- * Revision 1.6  2000/07/09 01:05:33  gerd
- * 	New methode 'close_all' that closes the clones, too.
- *
- * Revision 1.5  2000/07/08 16:24:56  gerd
- * 	Introduced the exception 'Not_resolvable' to indicate that
- * 'combine' should not try the next resolver of the list.
- *
- * Revision 1.4  2000/07/06 23:04:46  gerd
- * 	Quick fix for 'combine': The active resolver is "prefered",
- * but the other resolvers are also used.
- *
- * Revision 1.3  2000/07/06 21:43:45  gerd
- * 	Fix: Public(_,name) is now treated as System(name) if
- * name is non-empty.
- *
- * Revision 1.2  2000/07/04 22:13:30  gerd
- * 	Implemented the new API rev. 1.2 of pxp_reader.mli.
- *
- * Revision 1.1  2000/05/29 23:48:38  gerd
- * 	Changed module names:
- * 		Markup_aux          into Pxp_aux
- * 		Markup_codewriter   into Pxp_codewriter
- * 		Markup_document     into Pxp_document
- * 		Markup_dtd          into Pxp_dtd
- * 		Markup_entity       into Pxp_entity
- * 		Markup_lexer_types  into Pxp_lexer_types
- * 		Markup_reader       into Pxp_reader
- * 		Markup_types        into Pxp_types
- * 		Markup_yacc         into Pxp_yacc
- * See directory "compatibility" for (almost) compatible wrappers emulating
- * Markup_document, Markup_dtd, Markup_reader, Markup_types, and Markup_yacc.
- *
- * ======================================================================
- * Old logs from markup_reader.ml:
- *
- * Revision 1.3  2000/05/29 21:14:57  gerd
- * 	Changed the type 'encoding' into a polymorphic variant.
- *
- * Revision 1.2  2000/05/20 20:31:40  gerd
- * 	Big change: Added support for various encodings of the
- * internal representation.
- *
- * Revision 1.1  2000/03/13 23:41:44  gerd
- * 	Initial revision; this code was formerly part of Markup_entity.
- *
- *
- *)
